@@ -34,7 +34,7 @@ from . import DXDataObject
 from ..exceptions import DXFileError, DXIncompleteReadsError
 from ..utils import warn
 from ..utils.resolver import object_exists_in_project
-from ..compat import BytesIO, basestring, USING_PYTHON2
+from ..compat import BytesIO, basestring, USING_PYTHON2, md5_hasher
 
 
 DXFILE_HTTP_THREADS = min(cpu_count(), 8)
@@ -673,7 +673,7 @@ class DXFile(DXDataObject):
         if index is not None:
             req_input["index"] = int(index)
 
-        md5 = hashlib.md5()
+        md5 = md5_hasher()
         if hasattr(data, 'seek') and hasattr(data, 'tell'):
             # data is a buffer; record initial position (so we can rewind back)
             rewind_input_buffer_offset = data.tell()
@@ -709,14 +709,21 @@ class DXFile(DXDataObject):
         # The file upload API requires us to get a pre-authenticated upload URL (and headers for it) every time we
         # attempt an upload. Because DXHTTPRequest will retry requests under retryable conditions, we give it a callback
         # to ask us for a new upload URL every time it attempts a request (instead of giving them directly).
-        dxpy.DXHTTPRequest(get_upload_url_and_headers,
-                           data,
-                           jsonify_data=False,
-                           prepend_srv=False,
-                           always_retry=True,
-                           timeout=FILE_REQUEST_TIMEOUT,
-                           auth=None,
-                           method='PUT')
+        # APPS-650 - retries are given because part would sometimes stay in non-complete state. We retry to reupload the part in case this happens.
+        retries = 3
+        describe_input = {"fields": {"state": True}}
+
+        for i in range(retries):
+            dxpy.DXHTTPRequest(get_upload_url_and_headers,
+                               data,
+                               jsonify_data=False,
+                               prepend_srv=False,
+                               always_retry=True,
+                               timeout=FILE_REQUEST_TIMEOUT,
+                               auth=None,
+                               method='PUT')
+            if self._describe(self._dxid, describe_input, **kwargs).get('parts', {}).get(str(index), {}).get('state', "complete") == 'complete':
+                break
 
         self._num_uploaded_parts += 1
 
@@ -726,6 +733,10 @@ class DXFile(DXDataObject):
         if report_progress_fn is not None:
             report_progress_fn(self, len(data))
 
+    def wait_until_parts_uploaded(self, **kwargs):
+        self._wait_until_parts_uploaded(self, **kwargs)
+
+
     def get_download_url(self, duration=None, preauthenticated=False, filename=None, project=None, **kwargs):
         """
         :param duration: number of seconds for which the generated URL will be
