@@ -116,9 +116,13 @@ def response_iterator(request_iterator, thread_pool, max_active_tasks=None, do_f
 
     # The following two functions facilitate GC by not adding extra variables to the enclosing scope.
     def submit_task(task_iterator, executor, futures_queue):
-        task_callable, task_args, task_kwargs = next(task_iterator)
+        retval  = next(task_iterator, None)
+        if retval is None:
+            return False
+        task_callable, task_args, task_kwargs = retval
         task_future = executor.submit(task_callable, *task_args, **task_kwargs)
         futures_queue.append(task_future)
+        return True
 
     def next_result(tasks_in_progress):
         future = tasks_in_progress.popleft()
@@ -134,19 +138,13 @@ def response_iterator(request_iterator, thread_pool, max_active_tasks=None, do_f
         yield task_callable(*task_args, **task_kwargs)
 
     for _i in range(max_active_tasks):
-        try:
-            submit_task(request_iterator, thread_pool, tasks_in_progress)
-        except StopIteration:
+        retval = submit_task(request_iterator, thread_pool, tasks_in_progress)
+        if not retval:
             break
 
     while len(tasks_in_progress) > 0:
         result = next_result(tasks_in_progress)
-
-        try:
-            submit_task(request_iterator, thread_pool, tasks_in_progress)
-        except StopIteration:
-            pass
-
+        submit_task(request_iterator, thread_pool, tasks_in_progress)
         yield result
         del result
 
@@ -286,23 +284,6 @@ def json_loads_raise_on_duplicates(*args, **kwargs):
 def warn(*args, **kwargs):
     print(*args, file=sys.stderr, **kwargs)
 
-
-def instance_type_to_sys_reqs(instance_type, entrypoint="*"):
-    """
-    Returns a dictionary that can be passed as a "systemRequirements" input
-    to job/new or run/ API calls. The instance_type should be either a:
-    * string, eg. mem1_ssd1_x2
-    * dictionary, eg. {"main": "mem2_hdd2_x2", "other_function": "mem2_hdd2_x1"}
-    """
-    if isinstance(instance_type, basestring):
-        # By default, all entry points ("*") should use this instance type
-        return {entrypoint: {"instanceType": instance_type}}
-    elif isinstance(instance_type, dict):
-        # instance_type is a map of entry point to instance type
-        return {fn: {"instanceType": fn_inst} for fn, fn_inst in instance_type.items()}
-    else:
-        raise DXError('Expected instance_type field to be either a string or a dict')
-
 class Nonce:
     '''
     Generates a nonce by using the system's random number generator. If it fails
