@@ -53,6 +53,10 @@ def pick(choices, default=None, str_choices=None, prompt=None, allow_mult=False,
 
     At most one of allow_mult and more_choices should be set to True.
     '''
+    if len(choices) == 1:
+        choice = 0
+        return choice
+    
     for i in range(len(choices)):
         prefix = str(i) + ') '
         lines = choices[i].split("\n")
@@ -111,7 +115,7 @@ def paginate_and_pick(generator, render_fn=str, filter_fn=None, page_len=10, **p
             else:
                 if filter_fn(possible_next):
                     results.append(possible_next)
-                any_results = True
+            any_results = True
         if not any_results:
             return "none found"
         elif len(results) == 0:
